#/* Copyright 1998-2011 Research Foundation State University of New York */
#
#/* This file is part of QuB.                                            */
#
#/* QuB is free software; you can redistribute it and/or modify          */
#/* it under the terms of the GNU General Public License as published by */
#/* the Free Software Foundation, either version 3 of the License, or    */
#/* (at your option) any later version.                                  */
#
#/* QuB is distributed in the hope that it will be useful,               */
#/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
#/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
#/* GNU General Public License for more details.                         */
#
#/* You should have received a copy of the GNU General Public License,   */
#/* named LICENSE.txt, in the QuB program directory.  If not, see        */
#/* <http://www.gnu.org/licenses/>.                                      */

import QUB
from qubtree import Node
import CurveFitting
from math import *

# implements CurveFitting module api

class ExpoPlusCNLFitter(CurveFitting.NLFitter):
    def __init__(self):
        CurveFitting.NLFitter.__init__(self)
        self.initParams( 3,
                         ('Amp', 1.0, 1.0e-20, 1.0e20),
                         ('Tau', 1.0, -1.0e20, 1.0e20),
                         ('C', 1.0, 1.0e-20, 1.0e20) )
        
    def initComponent(self, param):
        pass

    def getEqn(self):
        return 'Amp * exp( -x / Tau ) + C'

    def minmaxX(self):
        minX = 1.0e10
        maxX = -1.0e10
        for x in self.X:
            if x < minX:
                minX = x
            if x > maxX:
                maxX = x
        return (minX, maxX)
    
    def setParamsFromData(self):
        minX, maxX = self.minmaxX()
        amp = max(self.Y)
        for param in self.params:
            param['Amp'].data[0] = amp

    def func(self, paramvals, x):
        return paramvals[0] * exp(- x / paramvals[1]) + paramvals[2]

    def deriv(self, paramvals, x):
        expL = exp(-x / paramvals[1])
        return [ expL,
                 x * paramvals[0] * expL / (paramvals[1] * paramvals[1]),
                 1 ]

    def onFitDone(self):
        pass

CurveFitting.fitters['Exponential plus C'] = ExpoPlusCNLFitter
